'use strict'

// command-line config
const yargs = require('yargs')

const ENV_STORAGE_URL = process.env.NSOLID_CONSOLE_STORAGE_URL
const ENV_CONSOLE_HOSTNAME = process.env.NSOLID_CONSOLE_HOSTNAME
const ENV_CONSOLE_PORT = process.env.NSOLID_CONSOLE_PORT || process.env.PORT

function getConfig (args) {
  const configDefaults = {
    storageUrl: ENV_STORAGE_URL || 'https://localhost:4000',
    storageAPIVersion: '/api/v1',
    consoleHostname: ENV_CONSOLE_HOSTNAME || '0.0.0.0',
    consolePort: ENV_CONSOLE_PORT || 6753
  }

  const tmp = yargs
  .usage('Starts the N|Solid console server\nUsage: $0 [options]\n')
  .describe('storageUrl', 'N|Solid storage server URL')
  .default('storageUrl', configDefaults.storageUrl)
  .alias('storageUrl', 's')
  .nargs('storageUrl', 1)
  .describe('consoleHostname', 'N|Solid console hostname')
  .default('consoleHostname', configDefaults.consoleHostname)
  .alias('consoleHostname', 'c')
  .nargs('consoleHostname', 1)
  .describe('consolePort', 'N|Solid console port')
  .default('consolePort', configDefaults.consolePort)
  .alias('consolePort', 'p')
  .nargs('consolePort', 1)
  .alias('help', 'h')
  .help('help')
  .parse(args)

  return Object.assign({}, configDefaults, {
    storageUrl: tmp.storageUrl,
    consoleHostname: tmp.consoleHostname,
    consolePort: tmp.consolePort
  })
}

module.exports = getConfig
