'use strict'

exports.handleRequest = handleRequest

const httpUtils = require('./http-utils')

// Handle the HTTP request
function handleRequest (req, res) {
  const context = req.nsStorage.context
  const eventStream = req.nsStorage.eventStream
  const interval = req.nsStorage.qData.interval
  const endDate = req.nsStorage.qData.end

  const app = req.nsStorage.qData.app

  poll(context, eventStream, app)

  httpUtils.syncPollTillDone(eventStream, interval, endDate,
    () => poll(context, eventStream, app)
  )
}

function poll (context, eventStream, app) {
  const res = context.vulnManager.getVulnsAndProcesses(app)

  const record = {
    time: Date.now(),
    app: app,
    hostCount: getHostCount(res.processes),
    processCount: res.processes.length,
    processes: res.processes,
    vulnerabilities: res.vulnerabilities,
    vulnerabilityCount: res.vulnerabilities.size
  }

  eventStream.writeData(record)
}

function getHostCount (procs) {
  const hosts = new Set()

  for (let proc of procs) {
    hosts.add(proc.hostname)
  }

  return hosts.size
}
