'use strict'

exports.handleRequest = handleRequest

const httpUtils = require('./http-utils')

// const util = require('../util')
// const Logger = util.getLogger(__dirname, __filename)

// Handle the HTTP request
function handleRequest (req, res) {
  const context = req.nsStorage.context
  const eventStream = req.nsStorage.eventStream
  const interval = req.nsStorage.qData.interval
  const endDate = req.nsStorage.qData.end

  poll(context, eventStream)

  httpUtils.syncPollTillDone(eventStream, interval, endDate,
    () => poll(context, eventStream)
  )
}

function poll (context, eventStream) {
  const applications = getApplications(context)

  const record = {
    time: Date.now(),
    applications: applications
  }

  eventStream.writeData(record)
}

// Get the application blobs to return
function getApplications (context) {
  const apps = new Map()

  // Collect all process counts and unique hosts.
  context.agentManager.list().forEach((agent) => {
    const info = agent.info
    if (info == null) return

    let app = apps.get(info.app)
    if (app == null) {
      app = {
        processCount: 0,
        vulnerabilityCount: 0,
        vulnerabilities: new Map(),
        name: info.app,
        hosts: new Set(),
        tags: info.tags || [],
        pid: info.pid
      }

      // Don't let this show up in the response.
      app.vulnerabilities.toJSON = () => undefined
      apps.set(info.app, app)
    }

    app.processCount++
    const vulnList = context.vulnManager.getVulnsByAgent(agent)

    vulnList.forEach(vuln => {
      app.vulnerabilities.set(`${vuln.packageName}:${vuln.title}`, vuln)
    })

    app.vulnerabilityCount = app.vulnerabilities.size
    app.hosts.add(info.hostname)
  })

  // Collapse unique hosts to host count
  const result = []
  for (let app of apps.values()) {
    app.hostCount = app.hosts.size
    app.hosts = Array.from(app.hosts)

    result.push(app)
  }

  return result
}
