'use strict'

exports.handleRequest = handleRequest

const lodash = require('lodash')

const util = require('../util')

const Logger = util.getLogger(__dirname, __filename)

// Handle the HTTP request
function handleRequest (req, res) {
  const context = req.nsStorage.context
  const eventStream = req.nsStorage.eventStream
  const qData = req.nsStorage.qData
  const diskAssetManager = context.diskAssetManager

  if (qData.start == null) qData.start = 0

  Logger.debug('gathering assets')
  getAllAssets(context, qData, gotAllAssets)

  // initial reply on query over assets
  function gotAllAssets (err, assets) {
    if (err) {
      Logger.error(err, 'error reading assets')

      eventStream.close()
      return
    }

    if (eventStream.isClosed()) return

    for (let asset of assets) {
      eventStream.writeData(assetWithoutFileName(asset))
    }

    // set up processing for future assets
    const checkEventStreamHandler = () => checkEventStream()
    const newAssetHandler = streamNewAsset

    const checkEventStreamInterval = setInterval(checkEventStreamHandler, 1000)
    checkEventStreamInterval.unref()
    diskAssetManager.addListener('add', newAssetHandler)

    // check if the event stream is closed
    function checkEventStream () {
      if (qData.end && Date.now() > qData.end) {
        eventStream.close()
      }

      if (eventStream.isClosed()) {
        clearInterval(checkEventStreamInterval)
        diskAssetManager.removeListener('add', newAssetHandler)
      }
    }

    function streamNewAsset (asset) {
      if (eventStream.isClosed()) return
      if (!diskAssetManager.queryMatchesAsset(qData, asset)) return

      eventStream.writeData(assetWithoutFileName(asset))
    }
  }
}

// Return an asset object without the fileName property
function assetWithoutFileName (asset) {
  return lodash.omit(asset, ['fileName'])
}

// Get all the assets available.
function getAllAssets (context, qData, cb) {
  const diskAssetManager = context.diskAssetManager
  if (diskAssetManager == null) return cb(new Error('diskAssetManager not initialized'))

  const assets = diskAssetManager.query(qData)
  setImmediate(() => cb(null, assets))
}
