'use strict'

exports.handleRequest = handleRequest

const util = require('../util')
const Logger = util.getLogger(__dirname, __filename)

// Handle the HTTP request
function handleRequest (req, res) {
  const context = req.nsStorage.context
  const params = {
    company: req.body.company,
    email: req.body.email,
    marketingOptIn: req.body.marketingOptIn,
    lastname: req.body.lastName,
    name: req.body.firstName,
    provider: context.provider.type,
    providerData: req.body.providerData || '',
    state: 'ACTIVE',
    trial: req.body.trial
  }

  Logger.debug(`handleRequest: provider: ${context.provider.type}, req.body: ${JSON.stringify(req.body)}`)

  // Concerning optOut, this API can only be used to set optOut to true.
  // the PUT API can set it to false and save a license to use.
  let optOut
  if (req.body.optOut === true || req.body.optOut === 'true') optOut = true
  if (req.body.optOut === false || req.body.optOut === 'false') optOut = false

  function responseHandler (action) {
    return function (err, license) {
      if (err) {
        res.status(400)
        res.send({ error: { message: `Could not ${action} license.` } })
        Logger.error(err, `Could not ${action} license.`)
        return
      }

      res.status(200)
      res.send(context.provider.license)
    }
  }

  if (optOut === false) {
    return responseHandler('update')(new Error('Valid license key is required if not opting out of connected features.'))
  }

  if ((params.email == null || params.email === '') && optOut !== true) {
    return responseHandler('create')(new Error('Email is required to create a license.'))
  }

  if (optOut === true) {
    return context.provider.setOptOut(optOut, responseHandler('update'))
  }

  return context.provider.createLicense(params, responseHandler('create'))
}
