'use strict'

exports.handleRequest = handleRequest

const util = require('../util')
const Logger = util.getLogger(__dirname, __filename)

// Handle the HTTP request
function handleRequest (req, res, err) {
  const context = req.nsStorage.context

  // check if body parser set an error
  if (req.nsStorage.bodyErr) {
    Logger.error(err, 'invalid JSON request body')
    return errorReturn(res, 'invalid JSON request body')
  }

  const oldWebhooks = context.webhooksManager.getConfig()
  const addWebhooks = req.body
  const newWebhooks = Object.assign({}, oldWebhooks, addWebhooks)

  // update the webhooks with the new data
  context.webhooksManager.setConfig(newWebhooks)

  res.status(200)
  res.send({time: Date.now()})
}

// Standard processing for an error return.
function errorReturn (res, message) {
  res.status(400)
  res.send({error: {
    message: message
  }})
}
