'use strict'

const fs = require('fs')

const util = require('./util')
const Logger = util.getLogger(__dirname, __filename)

const validStates = new Set(['ACTIVE', 'INACTIVE'])

const GLOBAL_OPT_OUT = (process.env.NSOLID_OPT_OUT != null)

const noLicense = {
  provider: 'default',
  optOut: GLOBAL_OPT_OUT,
  key: '',
  providerData: ''
}

class LicenseKey {
  constructor (provider, existing) {
    if (existing == null) {
      existing = Object.assign({}, noLicense)
      existing.provider = provider
    }
    this.optOut = !!existing.optOut || GLOBAL_OPT_OUT
    this.key = existing.key
    this.provider = existing.provider || provider
    this.providerData = existing.providerData
    this.firstName = existing.firstName
    this.lastName = existing.lastName
    this.email = existing.email
    this.marketingOptIn = existing.marketingOptIn
    this.company = existing.company
    this.expires = existing.expires
    this.trial = existing.trial
    this.keyState = existing.keyState
    this.recalculateValidity()
  }

  recalculateValidity () {
    this.valid = this.optOut || validStates.has(this.keyState)
  }

  save (filepath, cb) {
    Logger.debug(`save(${JSON.stringify(this)})`)

    this.recalculateValidity()

    const copy = Object.assign({}, noLicense, this)

    fs.writeFile(filepath, JSON.stringify(copy, null, 2), (err) => {
      if (err) {
        Logger.error(err, 'Could not save license file to disk.')
        return cb(err)
      }

      Logger.debug('Saved a new license file.', this)
      cb(null, this)
    })
  }
}

// creates a LicenseKey from a packet returned from the license service
LicenseKey.fromBody = function fromBody (provider, body) {
  Logger.debug('creating license key from server body', body)
  if (body == null) return new LicenseKey()

  const keyForm = {
    optOut: false, // if we're here we must not be in optOut...
    provider: body.provider || provider, // allow saved license to override display provider
    providerData: body.providerData,
    firstName: body.name,
    lastName: body.lastname,
    key: body.licensekey,
    email: body.email,
    marketingOptIn: body.marketingOptIn,
    company: body.company,
    expires: body.expires,
    trial: body.trial,
    keyState: body.state
  }
  return new LicenseKey(provider, keyForm)
}

LicenseKey.GLOBAL_OPT_OUT = GLOBAL_OPT_OUT

module.exports = LicenseKey
