'use strict'

module.exports = snapshot

const hyperquest = require('hyperquest')
const url = require('url')
const concat = require('concat-stream')
const json = require('../json')
const buildUrlObj = require('../build-url-obj')

const util = require('../../util')
const Logger = util.getLogger(__dirname, __filename)

function snapshot (appConfig, id, fn) {
  // NB: if you're going to return early for debugging, be sure to call
  // fn({ app: 'app id', id: 'process id', asset: 'asset id' })
  // so that you don't hang the queue

  // see comment in ./profile.js
  const urlObj = buildUrlObj(appConfig.snapshotEndpoint.url, 'snapshot', appConfig, { id: id })
  const snapshotUrl = url.format(urlObj)
  Logger.debug(`snapshot via ${snapshotUrl}`)

  hyperquest.post(snapshotUrl, (err, response) => {
    if (err) {
      return fn(err)
    }

    response.pipe(concat((payload) => {
      const obj = json(payload.toString())
      if (!obj) {
        return fn(new Error('Invalid response'))
      }

      if (obj.error) {
        return fn(new Error(obj.error.message) || 'Unknown error')
      }

      fn(null, obj)
    }))
  })
  .on('error', res => {
    fn(new Error('Invalid response:', res))
  })
  .end()
}
