#!/usr/bin/env node

'use strict'

// filter for messages from bole - https://www.npmjs.com/package/bole

const readline = require('readline')

const rl = readline.createInterface({ input: process.stdin })

rl.on('line', printLine)

function printLine (line) {
  const record = parse(line)
  if (!record) return console.log(line)

  const time = getLocalTime(record.time)
  const level = left(record.level.toUpperCase(), 5)
  const name = record.name
  const message = record.message

  const prefix = `${time} ${level} ${name}:`

  delete record.time
  delete record.hostname
  delete record.pid
  delete record.level
  delete record.name
  delete record.message

  const userObjectsFull = JSON.stringify(record, null, 4)
  const userObjects = userObjectsFull === '{}' ? '' : `\n${indent(userObjectsFull, 4)}`

  console.log(`${prefix} ${message}${userObjects}`)
}

function getLocalTime (isoDateString) {
  const isoDate = new Date(isoDateString)
  const offset = isoDate.getTimezoneOffset()
  const locDate = new Date(isoDate.getTime() - offset * 60 * 1000)
  return locDate.toISOString().substr(11, 8)
}

function indent (str, spaces) {
  return str.split('\n').map(line => `    ${line}`).join('\n')
}

function left (str, len) {
  str = `${str}`
  while (str.length < len) str = `${str} `
  return str
}

function parse (line) {
  let record

  try {
    record = JSON.parse(line)
  } catch (e) {
    return null
  }

  if (!record.time) return null
  if (!record.level) return null
  if (!record.name) return null
  if (!record.message) return null

  return record
}

/* {
  "time": "2016-02-04T13:46:41.312Z",
  "hostname": "hostname",
  "pid": 21156,
  "level": "debug",
  "name": "bole logger name",
  "message": "the message string",
  "userObject": {
    "userProp1": "blah",
    "userProp2": "zorg"
  }
} */
