'use strict'

const env = process.env.NS_ENV
const DYNAMODB_CUSTOMER_CONFIG_TABLE = {
  'Properties': {
    'AttributeDefinitions': [{
      'AttributeName': 'customerId',
      'AttributeType': 'S'
    }, {
      'AttributeName': 'webhookId',
      'AttributeType': 'S'
    }, {
      'AttributeName': 'event',
      'AttributeType': 'S'
    }],
    'KeySchema': [{
      'AttributeName': 'customerId',
      'KeyType': 'HASH'
    }, {
      'AttributeName': 'webhookId',
      'KeyType': 'RANGE'
    }],
    'LocalSecondaryIndexes': [{
      'IndexName': 'customerEvent',
      'KeySchema': [{
        'AttributeName': 'customerId',
        'KeyType': 'HASH'
      }, {
        'AttributeName': 'event',
        'KeyType': 'RANGE'
      }],
      'Projection': {
        'ProjectionType': 'ALL'
      }
    }],
    'ProvisionedThroughput': {
      'ReadCapacityUnits': 2,
      'WriteCapacityUnits': 2
    },
    'TableName': getCustomerConfigTable(env)
  }
}
const DYNAMODB_API_KEY_TABLE = {
  'Properties': {
    'AttributeDefinitions': [{
      'AttributeName': 'apiKey',
      'AttributeType': 'S'
    }],
    'KeySchema': [{
      'AttributeName': 'apiKey',
      'KeyType': 'HASH'
    }],
    'ProvisionedThroughput': {
      'ReadCapacityUnits': 2,
      'WriteCapacityUnits': 2
    },
    'TableName': getApiKeyTable(env)
  }
}

module.exports.DYNAMODB_API_KEY_TABLE = DYNAMODB_API_KEY_TABLE
module.exports.DYNAMODB_CUSTOMER_CONFIG_TABLE = DYNAMODB_CUSTOMER_CONFIG_TABLE
module.exports.getApiKeyTable = getApiKeyTable
module.exports.getCustomerConfigTable = getCustomerConfigTable

function getApiKeyTable (env) {
  return `${env}-webhooks-apikey`
}

function getCustomerConfigTable (env) {
  return `${env}-webhooks-customerconfig`
}
