'use strict'

exports.WebhooksCache = class WebhooksCache {
  constructor () {
    this.webhooks = {}
  }

  replace (params) {
    if (!params.customerId || !params.event || !params.webhooks) {
      throw new Error('customerId, event, and webhooks are required')
    }

    this.webhooks[params.customerId] = this.webhooks[params.customerId] || {}
    this.webhooks[params.customerId].ids = this.webhooks[params.customerId].ids || {}
    this.webhooks[params.customerId][params.event] = params.webhooks
    params.webhooks.forEach(wh => {
      this.webhooks[params.customerId].ids[wh.webhookId] = wh
    })
  }

  delete (params) {
    if (!params.customerId || !params.event) {
      throw new Error('customerId and event are required')
    }

    if (!this.webhooks[params.customerId] ||
        !this.webhooks[params.customerId][params.event] ||
        !this.webhooks[params.customerId][params.event].length) {
      return
    }

    if (this.webhooks[params.customerId].ids) {
      this.webhooks[params.customerId][params.event].forEach(wh => {
        delete this.webhooks[params.customerId].ids[wh.webhookId]
      })
    }
    delete this.webhooks[params.customerId][params.event]
  }

  getByEvent (params) {
    if (!params.customerId || !params.event) {
      throw new Error('customerId and event are required')
    }

    if (this.webhooks[params.customerId] && this.webhooks[params.customerId][params.event]) {
      return this.webhooks[params.customerId][params.event] || []
    }

    return []
  }

  get (params) {
    if (!params.customerId) {
      throw new Error('customerId is required')
    }

    if (!this.webhooks[params.customerId]) {
      return []
    }

    const eventNames = Object.keys(this.webhooks[params.customerId])
    if (!eventNames.length) {
      return []
    }
    return eventNames.reduce(function (total, cur) {
      if (this.webhooks[params.customerId][cur].length) {
        return total.concat(this.webhooks[params.customerId][cur])
      }
      return total
    }.bind(this), [])
  }

  getById (params) {
    if (!params.webhookId || !params.customerId) {
      throw new Error('webhookId and customerId is required')
    }

    if (!this.webhooks[params.customerId] ||
        !this.webhooks[params.customerId].ids[params.webhookId]) {
      return
    }

    return this.webhooks[params.customerId].ids[params.webhookId]
  }
}

exports.ApiKeysCache = class ApiKeysCache {
  constructor () {
    this.apiKeys = {}
  }

  // params.apiKey is the database object
  replace (params) {
    if (!params.apiKey) {
      throw new Error('apiKey is required')
    }

    this.apiKeys[params.apiKey.apiKey] = params.apiKey
  }

  // params.apiKey is the key string
  delete (params) {
    if (!params.apiKey) {
      throw new Error('apiKey is required')
    }

    delete this.apiKeys[params.apiKey]
  }

  // params.apiKey is the key string
  get (params) {
    if (!params.apiKey) {
      throw new Error('apiKey is required')
    }

    return this.apiKeys[params.apiKey] || null
  }
}
