'use strict'

const util = require('../util')
const Logger = util.getLogger(__dirname, __filename)
exports.handleRequest = handleRequest

function handleRequest (req, res) {
  const apiKey = req.headers['x-nsolid-api-key']
  const context = req.nsStorage.context
  const qData = req.nsStorage.qData

  context.apiKeys.validateApiKey({
    apiKey: apiKey
  }, (err, result) => {
    if (err || !result || !result.customerId) {
      return res.status(400).send({ error: 'API key validation error.' })
    }

    context.db.deleteWebhook({
      customerId: result.customerId,
      webhookId: qData.webhookId
    }, err => {
      if (err) {
        if (err.message.indexOf('required') > -1) {
          return res.status(400).send(util.formatError(err, context))
        }
        Logger.debug('Internal server error in DEL /webhook', qData, err)
        return res.status(500).send(util.formatError(err, context))
      }

      return res.status(200).send({ result: 'success' })
    })
  })
}
