'use strict'

const util = require('../util')
const Logger = util.getLogger(__dirname, __filename)

exports.handleRequest = handleRequest

function handleRequest (req, res) {
  const context = req.nsStorage.context
  const apiKey = req.headers['x-nsolid-api-key']

  context.apiKeys.validateApiKey({
    apiKey: apiKey
  }, (err, result) => {
    if (err || !result || !result.customerId) {
      return res.status(400).send({ error: 'API key validation error.' })
    }

    context.db.listWebhooks({
      customerId: result.customerId
    }, (err, webhooks) => {
      // we should get an empty array if there are really no webhooks
      if (err || !webhooks) {
        if (err.message.indexOf('required') > -1) {
          return res.status(400).send(util.formatError(err, context))
        }
        Logger.debug('Internal server error in GET /webhooks', err)
        return res.status(500).send(util.formatError(err, context))
      }

      return res.status(200).send(webhooks)
    })
  })
}
