'use strict'

const util = require('../util')
const Logger = util.getLogger(__dirname, __filename)

exports.handleRequest = handleRequest

function handleRequest (req, res) {
  const licenseKey = req.headers['x-nsolid-license-key']
  const context = req.nsStorage.context
  const qData = req.body

  if (!licenseKey) {
    const err = new Error('License validation error.')
    return res.status(400).send(util.formatError(err, context))
  }

  context.apiKeys.validateLicense({
    licenseKey: licenseKey
  }, (err, result) => {
    Logger.debug('license validation', licenseKey, err, result)

    // TODO don't allow the customer to set their own customerId!
    // Get this from the Accounts service!
    context.apiKeys.createKey({
      customerId: qData.customerId
    }, (err, apiKey) => {
      Logger.debug('key creation result', qData, err, apiKey)
      if (err) {
        if (err.message.indexOf('required') > -1) {
          const err = new Error('License validation error.')
          return res.status(400).send(util.formatError(err, context))
        }

        Logger.debug('Internal server error in POST /api-keys', qData, err)
        return res.status(500).send(util.formatError(err, context))
      }

      return res.status(200).send(apiKey)
    })
  })
}
