'use strict'

const util = require('../util')
const Logger = util.getLogger(__dirname, __filename)

const lodash = require('lodash')

exports.handleRequest = handleRequest

function handleRequest (req, res) {
  const apiKey = req.headers['x-nsolid-api-key']
  const qData = req.body
  const context = req.nsStorage.context
  const possibleParams = Object.keys(context.schemas.http.params.types)
  const params = lodash.pick(qData, possibleParams)

  context.apiKeys.validateApiKey({
    apiKey: apiKey
  }, (err, result) => {
    // context.config.web.proto
    if (err || !result || !result.customerId) {
      return res.status(400).send({ error: 'API key validation error.' })
    }

    params.customerId = result.customerId

    context.eventsManager.handleEvent(params, err => {
      Logger.debug('handleEvent finished', err)
      if (err) {
        if (err.message.indexOf('required') > -1) {
          return res.status(400).send(util.formatError(err, context))
        }
        Logger.debug('Internal server error in POST /events', qData, err)
        return res.status(500).send(util.formatError(err, context))
      }

      return res.status(200).send({ success: true })
    })
  })
}
