'use strict'

var path = require('path')

var bole = require('bole')

exports.getLogger = getLogger
exports.setLoglevel = setLoglevel
exports.getLogLevel = getLogLevel

let SavedLevel = 'info'

// Stuff to do when the module is initially loaded.
function onModuleLoad () {
  setLoglevel(process.env.LOGLEVEL || process.env.LOG_LEVEL || 'info')
}

// Return a new logger for a module, named with basenames of names passed in
// combined as a path; eg, (__dirname, __filename) -> 'lib/util.js'.
function getLogger (names) {
  let name = Array.prototype.slice.call(arguments)
    .map(name => path.basename(name))
    .join(path.sep)
  name = `webhooks-service:${name}`

  const logger = bole(name)
  logger.mute = mute
  logger.setLoglevel = setLoglevel
  logger.getLogLevel = getLogLevel

  return logger
}

// Set the logging level
function setLoglevel (level) {
  if (!level.match(/^(debug)|(info)|(warn)|(error)$/)) {
    level = 'info'
  }

  SavedLevel = level

  reset(SavedLevel)
}

function getLogLevel () {
  return SavedLevel
}

function mute (goSilent) {
  goSilent ? reset() : reset(SavedLevel)
}

function reset (level) {
  bole.reset()

  if (level == null) return

  bole.output({
    level: SavedLevel,
    stream: process.stdout
  })
}

// Run initial setup.
onModuleLoad()
