'use strict'

const util = require('./util')
const Logger = util.getLogger(__dirname, __filename)

let manager

exports.get = context => {
  if (manager) {
    return manager
  }

  manager = new WebhookEventsManager(context)
  return manager
}

class WebhookEventsManager {
  constructor (context) {
    this._context = context
  }

  handleEvent (params, cb) {
    cb = cb || (() => {})
    this._context.db.listEventWebhooks(params, (err, configs) => {
      if (err) {
        Logger.error(err)
        return cb(err)
      }

      Logger.debug(`${params.event} configs`, configs)
      configs.forEach(config => {
        this._context.queue.push(params, config, err => {
          if (err) {
            Logger.debug('A webhook failed.', params, config, err)
          }
        })
      })
      Logger.debug(`Successfully queued ${configs.length} tasks.`)
      cb()
    })
  }
}
