'use strict'

require('./lib/start-dynamodb')
const test = require('tape')
const uuid = require('uuid')
const testName = 'lib/apikeys.js'
const sinon = require('sinon')

let sandbox = sinon.sandbox.create()
const context = {
  hosted: true,
  env: 'dev',
  config: {
    db: {
      region: process.env.NSOLID_WEBHOOKS_DB_REGION || 'us-west-2',
      endpoint: process.env.NSOLID_WEBHOOKS_DB_ENDPOINT || 'http://localhost:8000'
    }
  }
}
context.db = require('../lib/db').get(context)

const apiKeys = context.apiKeys = require('../lib/apikeys').get(context)

function reset () {
  sandbox.restore()
  sandbox = sinon.sandbox.create()
}

test(`${testName} createKey with valid params`, t => {
  reset()

  apiKeys.createKey({
    customerId: uuid.v4()
  }, (err, key) => {
    t.notOk(err, 'No error is returned')
    t.ok(key.apiKey, 'apiKey is set')
    t.ok(key.customerId, 'customerId is set')
    t.ok(key.expiration, 'expiration is set')

    context.db.selectApiKey({ apiKey: key.apiKey }, (err, res) => {
      t.notOk(err, 'No error when selecting that key.')
      t.ok(res, 'Key is selectable from the database')
      t.equals(key.apiKey, res.apiKey, 'apiKey correct')
      t.equals(key.customerId, res.customerId, 'customerId is correct')
      t.equals(key.expiration, res.expiration, 'expiration is correct')
      t.end()
    })
  })
})

test(`${testName} createKey with invalid params`, t => {
  reset()
  sinon.spy(context.db._ddb, 'put')

  apiKeys.createKey({
    licenseKey: undefined
  }, (err, key) => {
    t.ok(err, 'An error is returned')
    t.notOk(key, 'No key is returned')
    t.equal(context.db._ddb.put.called, false, 'Database insert not called.')
    t.end()
  })
})
