'use strict'

const test = require('tape')
const testName = 'HTTP Action'
const httpAction = require('../lib/actions/http')
const nock = require('nock')

const exampleTask = {
  evt: {
    eventId: 'blah'
  }
}

test(`${testName} - Interval Generator 500 response`, t => {
  const generator = new httpAction.IntervalGenerator(exampleTask)
  const res = {
    statusCode: 500
  }
  generator.saveReport({
    statusCode: res.statusCode,
    res: res
  })
  const num = generator.getInterval(3)
  t.equals(num, Math.pow(30 * 3, 3), 'Interval follows the algorithm.')
  t.end()
})

test(`${testName} - Interval Generator 429 response with Retry-After`, t => {
  const generator = new httpAction.IntervalGenerator(exampleTask)
  const res = {
    statusCode: 429,
    headers: {
      'retry-after': 5
    }
  }
  generator.saveReport({
    statusCode: res.statusCode,
    res: res
  })
  const num = generator.getInterval(3)
  t.equals(num, 5000, 'Interval is 5 seconds.')
  t.end()
})

test(`${testName} - Interval Generator 429 response with no Retry-After`, t => {
  const generator = new httpAction.IntervalGenerator()
  const res = {
    statusCode: 429
  }
  generator.saveReport({
    statusCode: res.statusCode,
    res: res
  })
  const num = generator.getInterval(3)
  t.equals(num, Math.pow(30 * 3, 3), 'Interval follows the algorithm.')
  t.end()
})

test(`${testName} - run`, t => {
  const task = {
    config: {
      action: 'slack',
      url: 'https://example.tld/fake'
    },
    evt: {
      event: 'agent-online',
      app: 'my-app',
      agent: {
        hostname: 'server.example.tld',
        pid: 12345
      }
    }
  }
  nock('https://example.tld').post('/fake')
    .reply(200, {})

  httpAction.run({ task }, (err, report) => {
    t.error(err, 'No errors.')
    t.ok(report, 'Report is defined')
    t.ok(report.res, 'Response is defined')
    t.equals(report.res.statusCode, 200, 'statusCode is 200')
    t.end()
  })
})
