'use strict'

const path = require('path')
const ThisFile = path.join(path.basename(__dirname), path.basename(__filename))
const request = require('client-request')
const startServer = require('../start-server')
const integrationUtils = require('../lib/utils')

startServer.run(runTests)

function runTests (err, context) {
  const test = require('tape')
  if (err) throw err

  const config = context.config.web
  const apiURL = `${config.proto}://${config.server}/api`

  test(`${ThisFile} with no api key`, t => {
    const exampleEvent = {
      time: Date.now(),
      event: 'nsolid-agent-online',
      app: 'myapp'
    }
    const params = integrationUtils.getRequestParams({
      uri: `${apiURL}/v1/events`,
      method: 'post',
      body: exampleEvent,
      json: true
    })

    request(params, (err, res, body) => {
      t.error(err)

      t.equals(res.statusCode, 400, 'API responds 400')
      t.equals(typeof body, 'object', 'body is an object')
      t.equals(body.error.indexOf('validation') > -1, true,
        'License validation error.')
      t.end()
    })
  })

  test(`${ThisFile} with api key`, t => {
    const exampleEvent = {
      time: Date.now(),
      event: 'nsolid-agent-online',
      app: 'myapp'
    }
    const params = integrationUtils.getRequestParams({
      uri: `${apiURL}/v1/events`,
      method: 'post',
      body: exampleEvent,
      json: true,
      headers: {
        'x-nsolid-api-key': context.testApiKey.apiKey
      }
    })

    request(params, (err, res, body) => {
      t.error(err)

      t.equals(res.statusCode, 200, 'API responds 200')
      t.equals(typeof body, 'object', 'body is an object')
      t.equals(body.success, true, 'Success: true')
      t.end()
    })
  })
}
