'use strict'

const lodash = require('lodash')
const request = require('client-request')

const defaultRequestParams = {
  rejectUnauthorized: false,
  headers: {
    'Accept': 'application/json',
    'x-nsolid-api-key': ''
  }
}

exports.getRequestParams = function (params) {
  return lodash.defaults({}, params, defaultRequestParams)
}

//  Returns null or the Object into which the JSON was parsed.
exports.parseData = function (data) {
  let result = null
  data = data.toString()

  try {
    result = JSON.parse(data)
  } catch (err) {
    // This doesn't always have to be formatted as:
    //     data: {"some": "json"}
    // Start parsing for JSON after the first colon.
    try {
      result = JSON.parse(data.substring(data.indexOf(':') + 1, data.length))
    } catch (err2) {
      // if both JSON.parse calls throw, this function will just
      // return null.
    }
  }

  return result
}

exports.saveFile = (apiURL, agentId, cb) => {
  const params = exports.getRequestParams({
    uri: `${apiURL}/v1/snapshot?id=${agentId}`,
    method: 'post'
  })

  request(params)
    .on('response', res => {
      if (res.statusCode !== 201) {
        cb(new Error('Could not save file.'))
      }
    })
    .on('data', evt => {
      const parsed = exports.parseData(evt)
      if (parsed) {
        return cb(null, parsed)
      }
      cb(new Error('Response parse error.'))
    })
    .once('error', err => cb(err))
}
