'use strict'

const webhooks = require('../../nsolid-webhooks')
const uuid = require('uuid')
const ddbproc = require('../lib/start-dynamodb').ddb

let context

exports.run = function (cb) {
  if (context) {
    return process.nextTick(() => cb(null, context))
  }

  const config = webhooks.getConfig()

  config.web.server = incrementWebPort(config.web.server, 2)

  const bindings = {
    NSOLID_WEBHOOKS_WEB_SERVER: config.web.server,
    NSOLID_WEBHOOKS_DB_ENDPOINT: 'http://localhost:8000',
    NSOLID_WEBHOOKS_DB_REGION: null,
    NS_ENV: 'dev',
    NODE_TLS_REJECT_UNAUTHORIZED: '0',
    NSOLID_WEBHOOKS_FILE_NAME: './config/default-config.toml'
  }

  for (let key of Object.keys(bindings)) {
    process.env[key] = bindings[key]
  }

  webhooks.main((err, _context) => {
    if (err) {
      return cb(err)
    }

    context = _context
    context.testApiKey = {
      customerId: uuid.v4(),
      apiKey: uuid.v4(),
      expiration: Date.now() + 30 * 60 * 1000 // 30 mins
    }
    wrapApiValidation(context)

    cb(null, context)
  })
}

exports.shutdown = function () {
  ddbproc.kill()
  webhooks.shutdown()
}

function incrementWebPort (string, incr) {
  // url doesn't parse this format correctly.
  const pieces = string.split(':')
  pieces[1] = parseInt(pieces[1]) + incr
  return `${pieces[0]}:${pieces[1]}`
}

function wrapApiValidation (context) {
  const validateApiKey = context.apiKeys.validateApiKey.bind(context.apiKeys)
  context.apiKeys.validateApiKey = (params, cb) => {
    if (params.apiKey === context.testApiKey.apiKey) {
      setImmediate(() => cb(null, context.testApiKey))
      return
    }

    validateApiKey(params, cb)
  }
}
