'use strict'

const request = require('client-request')
const quasix = require('quasix-getopt')
const options = quasix.parse()
const async = require('async')

const licenseKey = 'f25a2e30-eb00-48c0-a74b-47c7a23b1bad'
const customerId = 'erintest1'
const webhook = 'http://24.96.15.188:8888'
let url
let apiKey

if (options.aws) {
  url = 'https://webhooks-staging.nsolid.nodesource.io/api/v1'
} else {
  url = 'http://localhost:80/api/v1'
}
const events = [
  'nsolid-agent-online', 'nsolid-agent-offline', 'nsolid-snapshot-generated',
  'nsolid-profile-generated', 'nsolid-vulnerability-detected',
  'nsolid-process-blocked'
]

async.series([
  getApiKey,
  deletePreviousWebhooks,
  createWebhooks
], err => {
  if (err) {
    console.error('Could not complete configuration.', err.message, err.stack)
    process.exit(1)
  }
})

function getApiKey (done) {
  const uri = `${url}/api-keys`
  console.log(`creating api key ${uri}`)
  request({
    uri: uri,
    method: 'POST',
    body: {
      customerId: customerId
    },
    headers: {
      'x-nsolid-license-key': licenseKey
    }
  }, (err, res, body) => {
    console.log(res.statusCode, (body || '').toString())
    if (err || !body || res.statusCode.toString().charAt(0) !== '2') {
      return done(err || new Error(`Error. ${res.statusCode}`))
    }
    if (body) {
      try {
        body = JSON.parse(body.toString())
      } catch (err) {
        return done(err)
      }

      if (body.apiKey) {
        apiKey = body.apiKey
        console.log('retrieved API key', apiKey)
        return done()
      }
      return done(new Error('Unknown error'))
    }
  })
}

function deletePreviousWebhooks (done) {
  const uri = `${url}/webhooks`
  console.log(`fetching webhooks ${uri}`)
  request({
    uri: uri,
    method: 'GET',
    headers: {
      'x-nsolid-api-key': apiKey
    }
  }, (err, res, body) => {
    let list
    if (err || !body || res.statusCode.toString().charAt(0) !== '2') {
      return done(err || new Error(`Error. ${res.statusCode}`))
    }
    if (body) {
      try {
        list = JSON.parse(body.toString())
      } catch (err) {
        return done(err)
      }

      if (list.length || list.length != null) {
        async.parallel(list.map(deleteWebhook), done)
        return
      }
    }
    return done(new Error('Unknown error'))
  })
}

function deleteWebhook (webhook) {
  return function (done) {
    const uri = `${url}/webhook/${webhook.webhookId}`
    console.log(`deleting webhook ${uri}`)
    request({
      uri: uri,
      method: 'DELETE',
      headers: {
        'x-nsolid-api-key': apiKey
      }
    }, (err, res, body) => {
      console.log(res.statusCode, (body || '').toString())
      if (err || res.statusCode.toString().charAt(0) !== '2') {
        return done(err || new Error(`Error. ${res.statusCode}`))
      }

      console.log('Deleted previous webhook', webhook.webhookId)
      done()
    })
  }
}

function createWebhooks (done) {
  async.parallel(events.map(createWebhook), done)
}

function createWebhook (eventName) {
  return function (done) {
    request({
      uri: `${url}/webhooks`,
      method: 'POST',
      body: {
        event: eventName,
        url: webhook
      },
      headers: {
        'x-nsolid-api-key': apiKey
      }
    }, (err, res, body) => {
      if (err || !body || res.statusCode.toString().charAt(0) !== '2') {
        return done(err || new Error(`Error. ${res.statusCode}`))
      }
      if (body) {
        try {
          body = JSON.parse(body.toString())
        } catch (err) {
          return done(err)
        }

        console.log(`Webhook creation complete for event '${body.event}'.`)
        done()
      }
    })
  }
}
