'use strict'
// builds the /lib/config-default.js file based on /config-sample.toml

const fs = require('fs')
const path = require('path')

const toml = require('toml')

const PROGRAM = `${path.basename(__filename)}`

const TomlFile = path.join(__dirname, '..', 'config-sample.toml')
const JsFile = path.join(__dirname, '..', 'lib', 'config-default.js')

let tomlConfigSource = fs.readFileSync(TomlFile, 'utf8')

try {
  tomlConfigSource = fs.readFileSync(TomlFile, 'utf8')
} catch (err) {
  console.log(`error reading TOML source file ${TomlFile}`)
  process.exit(1)
}

let tomlConfig

try {
  tomlConfig = toml.parse(tomlConfigSource)
} catch (err) {
  console.log(`error parsing TOML source file ${TomlFile}:`, err)
  process.exit(1)
}

const jsConfig = `'use strict'

// config file built
//   by: ${PROGRAM}
//   on: ${new Date()}

// DO NOT EDIT / CHANGE THIS FILE!!!
//
// To use a custom configuration for nsolid-storage, copy and then edit the
// file ../config-sample.toml instead; instructions for using the configuration
// file are included in the sample configuration file.

// disable 'standard' errors on indentation and double quote usage

/* eslint-disable indent, quotes */

module.exports = ${JSON.stringify(tomlConfig, null, 4)}
`

try {
  fs.writeFileSync(JsFile, jsConfig)
} catch (err) {
  console.log(`error writing config to ${JsFile}: ${err}`)
  process.exit(1)
}

console.log(`wrote config to ${JsFile}`)
