'use strict'

const http = require('http')
const WebhooksManager = require('./webhooks-manager')
const fs = require('fs')
const sentMessages = []
const rxMessages = []
const startTime = Date.now()
let rxCount = 0
let txCount = 0
let MESSAGES_PER_SEC = 1000
let SEND_MESSAGES_LIMIT = 10000

// Simple webhook receiver server
// Use ngrok or something similar to get a public URI
const server = http.createServer((req, res) => {
  let body = ''

  req.on('data', buf => {
    body += buf.toString()
  })

  req.on('end', () => {
    try {
      body = JSON.parse(body)
    } catch (err) {
      console.error('Parse error.', body)
      return
    }

    handle(body)
  })
  res.writeHead(204, 'No Content')
  res.end()
})
server.on('clientError', (err, socket) => {
  console.error(err)
  socket.end('HTTP/1.1 400 Bad Request\r\n\r\n')
})
server.listen(8888)
console.log('Server started. http://localhost:8888')

const man = new WebhooksManager()
man.on('ready', () => {
  let sendTimer = setInterval(() => {
    const msg = man.send(txCount)
    sentMessages.push(msg)
    // console.log('TX', msg.id)
    txCount++
  }, Math.floor(1000 / MESSAGES_PER_SEC))

  setInterval(() => {
    console.log(`${txCount} sent and ${rxCount} received. Waiting on ${sentMessages.length}. Start: ${startTime} Now: ${Date.now()}`)
    if (txCount > SEND_MESSAGES_LIMIT && sendTimer) {
      // console.log('still waiting on', sentMessages)
      clearInterval(sendTimer)
      sendTimer = null
    }
  }, 1000)
})

function handle (evt) {
  // console.log('RX', evt.id)
  const tx = evt.txTime
  const rx = Date.now()
  const time = rx - tx

  const index = sentMessages.findIndex(msg => msg.id === evt.id)
  if (index > -1) {
    const msg = sentMessages[index]
    msg.rxTime = rx
    msg.flightTime = time
    delete msg.event
    delete msg.apiKey
    delete msg.id
    // rxMessages.push(msg)
    sentMessages.splice(index, 1)
    rxCount++
  } else {
    console.log('****** did not find', evt)
    return
  }
}

process.on('SIGINT', () => {
  fs.writeFileSync('./data.json', JSON.stringify(rxMessages, null, 4))
  process.exit()
})
