'use strict'

const cli = require('./cli')

var through2 = require('through2')
var JSONstream = require('json-stream')

module.exports.rawFormat = through2.ctor()
module.exports.fromJsonLines = fromJsonLines
module.exports.toJsonLines = toJsonLines
module.exports.Consolize = Consolize
module.exports.fetchAsset = fetchAsset

function fromJsonLines () {
  // convert from newline-delimited JSON to object stream
  return JSONstream()
}

function toJsonLines () {
  // stringify each record and print as rows (i.e. opposite of fromJsonLines)
  return through2.obj(function _write (chunk, encoding, callback) {
    this.push(JSON.stringify(chunk) + '\n')
    callback()
  })
}

// Add a final \n if we're in a TTY and it doesn't have one already
function Consolize (output) {
  var final = null
  return through2(function _noop (chunk, encoding, callback) {
    final = chunk[chunk.length - 1]
    callback(null, chunk)
  }, function _addNewline (callback) {
    if (output.isTTY) {
      if (final !== null && final !== 10) { // 0x10 = \n
        // only add newline if in TTY & doesn't already end with one
        this.push('\n')
      }
    }
    callback()
  })
}

function fetchAsset (needsAsset, remoteSocket, origStream) {
  var chunks = []
  return through2(function (chunk, encoding, callback) {
    if (needsAsset) {
      chunks.push(chunk)
      callback()
    } else {
      callback(null, chunk)
    }
  }, function (callback) {
    if (needsAsset) {
      let json = chunks.join('')
      var parsed = JSON.parse(json)
      if (parsed.error) {
        this.push(json)
        callback()
      } else if (parsed.asset) {
        var assetOut = cli('asset', remoteSocket, { 'asset': parsed.asset })
        assetOut.on('error', (err) => { origStream.emit('error', err) })
        assetOut.on('data', (data) => { this.push(data) })
        assetOut.on('end', () => { callback() })
      } else {
        origStream.emit('error', new Error('asset not found'))
        callback()
      }
    } else {
      callback()
    }
  })
}
