'use strict'

const concat = require('concat-stream')
const debug = require('util').debuglog('cli')

module.exports = function getLimit (client, opts, cb) {
  function onerror (err) {
    cb(new Error(`Encountered error during 'ls' while trying to determine limit: ${err.toString}`))
  }

  function onresponse (res) {
    if (!res) return cb(null, 0)
    // count the lines of the response (it will have one per connected agent)
    const limit = res.toString().trim('\n').split('\n').length
    cb(null, limit)
  }

  debug('determining limit via `ls`')

  // use same options as supplied command but don't `force`
  const listOpts = Object.assign({}, opts, { force: false })
  client.read('list', listOpts)
    .on('error', onerror)
    .pipe(concat(onresponse))
    .on('error', onerror)
}
